/*
Copyright 2011 Johan Maasing

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
 */
package nu.zoom.catonine.prefs.gui.impl;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JComboBox;
import javax.swing.JLabel;

import nu.zoom.catonine.prefs.Preferences;
import nu.zoom.catonine.prefs.Preferences.WindowOpenMode;
import nu.zoom.catonine.prefs.gui.PreferencesGUI;
import nu.zoom.gal.error.ErrorReporter;
import nu.zoom.swing.desktop.Workbench;
import nu.zoom.swing.desktop.WorkbenchFrame;
import nu.zoom.swing.layout.VerticalPanel;
import nu.zoom.ui.Resources;
import nu.zoom.ui.Resources.ResourceNotFoundException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * @author "Johan Maasing" &lt;johan@zoom.nu&gt;
 * 
 */
public class PreferencesGUIFactory implements PreferencesGUI {
	private final Log log = LogFactory.getLog(getClass());
	private final Workbench workbench;
	private final Resources resources;
	private final ErrorReporter errorReporter;
	private final Preferences preferences;
	private WindowModeItems[] items;

	public PreferencesGUIFactory(final Workbench workbench, final Resources resources,
			final ErrorReporter errorReporter, final Preferences preferences) {
		super();
		this.workbench = workbench;
		this.resources = resources;
		this.errorReporter = errorReporter;
		this.preferences = preferences;
	}

	@Override
	public WorkbenchFrame displayPreferences() {
		final VerticalPanel panel = new VerticalPanel();
		final WorkbenchFrame workbenchFrame =
				workbench.createWorkbenchFrame("svansprogram-preferences-window", panel, true, true);
		try {
			final JComboBox detachedControl = new JComboBox(getWindowItems());
			detachedControl.setEditable(false);
			detachedControl.addActionListener(new ActionListener() {

				@Override
				public void actionPerformed(final ActionEvent event) {
					Object selectedItem = detachedControl.getSelectedItem();
					if ((selectedItem != null) && (selectedItem instanceof WindowModeItems)) {
						WindowOpenMode newMode = ((WindowModeItems) selectedItem).getMode();
						try {
							log.debug("Setting window open mode to: " + newMode);
							preferences.setWindowOpenMode(newMode);
						} catch (final Exception exc) {
							errorReporter.reportError(exc);
						}
					}
				}
			});
			switch (this.preferences.getWindowOpenMode()) {
				case Attached:
					detachedControl.setSelectedItem(this.items[1]);
					break;
				case Detached:
					detachedControl.setSelectedItem(this.items[2]);
				default:
					break;
			}
			JLabel wmodeLabel = new JLabel(resources.getMessage("nu.zoom.catonine.preferences.wmode"));
			panel.addRow(wmodeLabel, detachedControl);
			wmodeLabel.setToolTipText(resources.getMessage("nu.zoom.catonine.preferences.wmode.tt"));
			workbenchFrame.setTitle(resources.getMessage("nu.zoom.catonine.preferences.menu"));
			workbenchFrame.setFrameIcon(resources.getIcon("nu.zoom.catonine.preferences.menu.icon"));
		} catch (Exception e) {
			errorReporter.reportError(e);
		}
		return workbenchFrame;
	}

	private synchronized WindowModeItems[] getWindowItems() throws ResourceNotFoundException {
		if (this.items == null) {
			this.items = new WindowModeItems[3];
			this.items[0] =
					new WindowModeItems(WindowOpenMode.Rememeber,
							this.resources.getMessage("nu.zoom.catonine.preferences.wmode.rememeber.lm"));
			this.items[1] =
					new WindowModeItems(WindowOpenMode.Attached,
							this.resources.getMessage("nu.zoom.catonine.preferences.wmode.attached.lm"));
			this.items[2] =
					new WindowModeItems(WindowOpenMode.Detached,
							this.resources.getMessage("nu.zoom.catonine.preferences.wmode.detached.lm"));
		}
		return this.items;
	}

	static class WindowModeItems {
		final private WindowOpenMode mode;
		final private String message;

		public WindowModeItems(final WindowOpenMode mode, final String message) {
			super();
			this.mode = mode;
			this.message = message;
		}

		@Override
		public String toString() {
			return message;
		}

		WindowOpenMode getMode() {
			return mode;
		}

	}
}
